package wssl.aggregator;

import java.io.IOException;
import java.net.URL;
import java.util.AbstractMap;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.parsers.ParserConfigurationException;

import org.jbpm.ruleflow.core.RuleFlowProcess;
import org.jbpm.workflow.core.node.Split;
//import org.jbpm.workflow.core.Node;
import org.jbpm.workflow.core.node.StartNode;
import org.kie.api.KieBase;
import org.kie.api.definition.process.Connection;
import org.kie.api.definition.process.Node;
import org.kie.api.definition.process.Process;
import org.kie.api.io.ResourceType;
import org.kie.internal.builder.KnowledgeBuilder;
import org.kie.internal.builder.KnowledgeBuilderFactory;
import org.kie.internal.io.ResourceFactory;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import wssl.parser.DOMParser2;


/**
 * Implements a QoS Aggregator application for a given BPMN2 process
 * @author George Baryannis
 */
public class QosAggregator {
	
	public enum AggMode {BOOLEAN, ORDERED, UNORDERED, REPUTATION, REPMINMAX, THROUGHPUT, COST, AVAILABILITY, TIME}
	static int h=0;

	public static void main(String[] args) {
		long startTime = System.currentTimeMillis();
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
		//kbuilder.add(ResourceFactory.newClassPathResource("Sequence1.bpmn"), ResourceType.BPMN2);
		kbuilder.add(ResourceFactory.newClassPathResource("AND-AND2.bpmn"), ResourceType.BPMN2);
		long midTime = System.currentTimeMillis();
		//kbuilder.add(ResourceFactory.newClassPathResource("OR-OR.bpmn"), ResourceType.BPMN2);
		//kbuilder.add(ResourceFactory.newClassPathResource("XOR-XOR.bpmn"), ResourceType.BPMN2);
		//kbuilder.add(ResourceFactory.newClassPathResource("AND-Disc.bpmn"), ResourceType.BPMN2);
		//kbuilder.add(ResourceFactory.newClassPathResource("OR-Disc.bpmn"), ResourceType.BPMN2);
		//kbuilder.add(ResourceFactory.newClassPathResource("DetLoop.bpmn"), ResourceType.BPMN2);
		//kbuilder.add(ResourceFactory.newClassPathResource("Complex.bpmn"), ResourceType.BPMN2);
		//read QoS type from main args, then determine the mode
		String attrName = "owlq:ExecutionTime";
		AggMode mode = AggMode.TIME;
		KieBase kbase = kbuilder.newKnowledgeBase();
		Collection<Process> procList = kbase.getProcesses();
		for (Process proc : procList) {
			System.out.println(proc.getName());
			RuleFlowProcess rf = (RuleFlowProcess) proc;
			float aggrValues[] = {Integer.MAX_VALUE, Integer.MIN_VALUE};
			Map<Integer, String> aggrSet = new HashMap<Integer,String>();
			aggrSet.put(Integer.MAX_VALUE,"MAXIMUM");
			if(mode!=AggMode.COST && mode!=AggMode.AVAILABILITY & mode!=AggMode.TIME){
				Node[] nodeList = rf.getNodes();
				nodeloop:
				for(Node node : nodeList){
					String curNodeName = node.getClass().getName(); 
					if(curNodeName.endsWith("ItemNode")){
						Float val = (Float) getQoSValue(node.getName(), attrName);		//we assume that operands between offering and demand match
						if(val==null){
							System.out.println("Could not find attribute " + attrName + " in specification " + node.getName() + ".wssl");
							System.exit(-1);
						}
						Boolean valB = false;
						Map.Entry<Integer,String> valM= new AbstractMap.SimpleEntry<>(2, "MEDIUM");
						Map<Integer, String> valUM = new HashMap<Integer,String>();
						valUM.put(1, "FAILURE");
						valUM.put(2, "OMISSION");
						switch (mode){
						case BOOLEAN:
							if(valB==false){
								aggrValues[0] = 0;
							}
							break;
						case ORDERED:		//QoS values must be of the Map.Entry<Integer,String> type
							Map.Entry<Integer,String> aggrVal = aggrSet.entrySet().iterator().next();
							if(valM.getKey() < aggrVal.getKey()){
								aggrSet.remove(aggrVal.getKey());
								aggrSet.put(valM.getKey(), valM.getValue());
							}
							break;
						case UNORDERED:		//QoS values must be of the Map<Integer, String> type
							if(aggrSet.isEmpty()){
								break nodeloop;
							}
							else if(aggrSet.containsKey(Integer.MAX_VALUE)){	//first iteration
								aggrSet = valUM;
							}
							else{
								Set<Integer> keySet = aggrSet.keySet();
								for(Integer key : keySet){
									if(!valUM.containsKey(key)){
										aggrSet.remove(key);
									}
								}
							}
							break;							
						case REPUTATION:		//average
							if(aggrValues[0] == Integer.MAX_VALUE){
								aggrValues[0]=0;
								aggrValues[1]=0;
							}
							aggrValues[0]++;
							aggrValues[1]+=val;
							break;
						case REPMINMAX:		//min-max
							if(aggrValues[0] == Integer.MAX_VALUE){ 	//initialization
								aggrValues[0] = val;
								aggrValues[1] = val;
							}
							else{
								if(val<aggrValues[0]){
									aggrValues[0] = val;
								}
								if(val>aggrValues[1]){
									aggrValues[1] = val;
								}
							}
							break;							
						case THROUGHPUT:
							if(val<aggrValues[0]){
								aggrValues[0] = val;
							}
							break;
						default:
							break;
						
						}
					}
				}
			}
			else{
				StartNode start = rf.getStart(null);
				//Map<String, List<Connection>> InC = start.getIncomingConnections();
				//Map<String, List<Connection>> OutC = start.getOutgoingConnections();
				List<Connection> OutC = start.getDefaultOutgoingConnections();
				//OutC = OutC.get(0).getTo().getOutgoingConnections("DROOLS_DEFAULT");
				Node curNode = OutC.get(0).getTo();		//a StartNode always has at least one connection
				aggrValues = AggregatePath(curNode, mode, attrName);
			}
			switch(mode){
			case BOOLEAN:
				if(aggrValues[0]==0){
					System.out.println("The aggregated value for exception handling, for composite process " + proc.getName() + " is False");
				}
				else{
					System.out.println("The aggregated value for exception handling, for composite process " + proc.getName() + " is True");
				}
				break;			
			case AVAILABILITY:
				System.out.println("The aggregated value for availability, for composite process " + proc.getName() + " is: " + aggrValues[0]);
				break;
			case COST:
				if(aggrValues[0]==aggrValues[1]){
					System.out.println("The aggregated value for cost, for composite process " + proc.getName() + " is: " + aggrValues[0]);
				}
				else{
					System.out.println("The aggregated value range for cost, for composite process " + proc.getName() + " is: " + aggrValues[0] + " to " + aggrValues[1]);
				}
				break;
			case ORDERED:
				System.out.println("The aggregated value for robustness, for composite process " + proc.getName() + " is: " + aggrSet.entrySet().iterator().next().getValue());
				break;
			case UNORDERED:
				if(aggrSet.isEmpty()){
					System.out.println("The aggregated value for failure masking, for composite process " + proc.getName() + " is the empty set");
				}
				else{
					String pr = "The aggregated value for failure masking, for composite process " + proc.getName() + " is the set {";
					for(String s : aggrSet.values()){
						pr += s + ", ";
					}
					System.out.println(pr.substring(0, pr.length()-2) +"}");
				}
				break;				
			case REPUTATION:
				System.out.println("The aggregated value for reputation, for composite process " + proc.getName() + " is: " + aggrValues[1]/aggrValues[0]);
				break;
			case REPMINMAX:
				if(aggrValues[0]==aggrValues[1]){
					System.out.println("The aggregated value for reputation, for composite process " + proc.getName() + " is: " + aggrValues[0]);
				}
				else{
					System.out.println("The aggregated value range for reputation, for composite process " + proc.getName() + " is: " + aggrValues[0] + " to " + aggrValues[1]);
				}
				break;				
			case THROUGHPUT:
				System.out.println("The aggregated value for throughput, for composite process " + proc.getName() + " is: " + aggrValues[0]);
				break;
			case TIME:
				if(aggrValues[0]==aggrValues[1]){
					System.out.println("The aggregated value for execution time, for composite process " + proc.getName() + " is: " + aggrValues[0]);
				}
				else{
					System.out.println("The aggregated value range for execution time, for composite process " + proc.getName() + " is: " + aggrValues[0] + " to " + aggrValues[1]);
				}
				break;

			default:
				break;
			
			}
	        long stopTime = System.currentTimeMillis();
	        long elapsedTime = stopTime - startTime;
	        long midelapsedTime = midTime - startTime;
	        System.out.println(midelapsedTime);
	        System.out.println(elapsedTime);
	    }
	}
	
    /**
     * Returns the aggregated value for the sequence that starts with curNode
     * and ends at an EndNode or a Split.
     * @param curNode the starting node for the aggregation
     * @param mode the aggregation mode, depending on the nature of the QoS attribute
     * @param attrName the QoS attribute name
     * @return the aggregated value
     */
	static float[] AggregatePath(Node curNode, AggMode mode, String attrName){
		ArrayList<Float> pathValues = new ArrayList<Float>();
		ArrayList<Float> pathValuesAlt = new ArrayList<Float>();	//to express range (COST, TIME)
		String curNodeType = curNode.getClass().getName();
		Node startNode = curNode.getIncomingConnections("DROOLS_DEFAULT").get(0).getFrom();		//determine if the path is a Start-End or a Split-Join one
		String startNodeType = startNode.getClass().getName();
		while((startNodeType.endsWith("StartNode") && !curNodeType.endsWith("EndNode")) || 
			   startNodeType.endsWith("Split") && !curNodeType.endsWith("Join")){
			//System.out.println(curNodeType + " " + startNodeType);
			//System.out.println(curNodeType);
			String curNodeName = curNode.getName();
			//System.out.println(curNodeName);
			if(curNodeType.endsWith("WorkItemNode")){	//Loop or Sequence
				int loopMax = 1;
				if(curNodeName.startsWith("Loop")){		//Deterministic Loop
					String[] parts = curNodeName.split("_");
					loopMax = Integer.parseInt(parts[1]);
					curNodeName = parts[2];
				}
				/*to be implemented: parse WSSL file for the curNodeName service and fetch the
				 * value for the QoS attribute we're aggregating - DONE?
				 */
				//float val = 1;
				Float val = (Float) getQoSValue(curNode.getName(), attrName);
				if(val==null){
					System.out.println("Could not find attribute " + attrName + " in specification " + curNode.getName() + ".wssl");
					System.exit(-1);
				}
				h++;
				if(mode == AggMode.AVAILABILITY){
					float loopedVal = val;
					for(int k=1; k<loopMax; k++){
						loopedVal *= val;
					}
					pathValues.add(loopedVal);
					pathValuesAlt.add(loopedVal);
				}
				else{
					pathValues.add(loopMax*val);
					pathValuesAlt.add(loopMax*val);					
				}
				curNode = curNode.getOutgoingConnections("DROOLS_DEFAULT").get(0).getTo();
				curNodeType = curNode.getClass().getName();
				continue;
			}
			else if(curNodeType.endsWith("Split")){
				List<Connection> OutC = curNode.getOutgoingConnections("DROOLS_DEFAULT");
				float valArray[][] = new float[OutC.size()][2];
				for(int i=0; i<OutC.size(); i++){
					valArray[i] = AggregatePath(OutC.get(i).getTo(), mode, attrName);
				}
				Split splitNode = (Split) curNode;
				float val=0, valAlt=0;
				switch (mode) {						//aggregation is based on attribute type and composition pattern
				case COST:
					switch (splitNode.getType()) {
					case Split.TYPE_AND:
						for(int j=0; j<OutC.size(); j++){
							val+=valArray[j][0];
							valAlt+=valArray[j][1];
						}
						pathValues.add(val);
						pathValuesAlt.add(valAlt);
						break;
					case Split.TYPE_OR:
					case Split.TYPE_XOR:
						val = valArray[0][0];
						valAlt = valArray[0][1];
						for(int j=1; j<OutC.size(); j++){
							if(valArray[j][0]<val){
								val = valArray[j][0];
							}
							if(valArray[j][1]>valAlt){
								valAlt = valArray[j][1];
							}
						}
						pathValues.add(val);
						pathValuesAlt.add(valAlt);
						break;
					}
					break;
				case AVAILABILITY:
					switch (splitNode.getType()) {
					case Split.TYPE_AND:
						val = valArray[0][0];
						for(int j=1; j<OutC.size(); j++){
							val*=valArray[j][0];
						}
						pathValues.add(val);
						pathValuesAlt.add(val);
						break;
					case Split.TYPE_OR:
					case Split.TYPE_XOR:
						val = valArray[0][0];
						for(int j=1; j<OutC.size(); j++){
							if(valArray[j][0]<val){
								val = valArray[j][0];
							}
						}
						pathValues.add(val);
						pathValuesAlt.add(val);
						break;
					}
					break;
				case TIME:
					switch (splitNode.getType()) {
					case Split.TYPE_AND:
						val = valArray[0][0];
						for(int j=1; j<OutC.size(); j++){
							if(valArray[j][0]>val){
								val = valArray[j][0];
							}
						}
						pathValues.add(val);
						pathValuesAlt.add(val);
						break;
					case Split.TYPE_OR:
					case Split.TYPE_XOR:
						val = valArray[0][0];
						valAlt = valArray[0][1];
						for(int j=1; j<OutC.size(); j++){
							if(valArray[j][0]<val){
								val = valArray[j][0];
							}
							if(valArray[j][1]>valAlt){
								valAlt = valArray[j][1];
							}
						}
						pathValues.add(val);
						pathValuesAlt.add(valAlt);
						break;
					}					
					break;
				default:
					break;
				
				}
				int splitCount=0;
				while(!curNodeType.endsWith("Join") || splitCount!=0){			//get to the end of the Split/Join by traversing the first path
					if(curNodeType.endsWith("Join")){			//the internal parallelity has been overcome
						splitCount--;
					}
					curNode = curNode.getOutgoingConnections("DROOLS_DEFAULT").get(0).getTo();
					curNodeType = curNode.getClass().getName();
					if(curNodeType.endsWith("Split")){			//there's an internal parallelity that needs to be overcome
						splitCount++;
					}
				}
				curNode = curNode.getOutgoingConnections("DROOLS_DEFAULT").get(0).getTo();
				curNodeType = curNode.getClass().getName();
			}		
		}
		float val=0, valAlt=0;
		switch (mode){
		case COST:
		case TIME:
			for(int i=0;i<pathValues.size();i++){
				val+=pathValues.get(i);
			}
			for(int i=0;i<pathValuesAlt.size();i++){
				valAlt+=pathValuesAlt.get(i);
			}
			break;
		case AVAILABILITY:
			val = pathValues.get(0);
			valAlt = pathValuesAlt.get(0);
			for(int i=1;i<pathValues.size();i++){
				val*=pathValues.get(i);
			}
			for(int i=1;i<pathValuesAlt.size();i++){
				valAlt*=pathValuesAlt.get(i);
			}
			break;
		default:
			break;
		}
		float ret[] = {val, valAlt};
		return ret;
	}
	
    /**
     * Returns the value for the associated QoS attribute and service, for the default QoS profile
     * @param serviceName the name of the service (must be the same as the corresponding wssl file name
     * @param attrName the name of the QoS attribute
     * @return the QoS value, or null if the attribute was not found in the default profile (or if a
     * specification file for the particular service does not exist
     */
	static Object getQoSValue(String serviceName, String attrName){
		URL def_location = QosAggregator.class.getProtectionDomain().getCodeSource().getLocation();
		Object retValue = null;
        //String url = "test3.wssl";
		String url = serviceName + ".wssl";
        String url_s = "wssl3.xsd";		
        DOMParser2 pr = new DOMParser2();
        try {
			if(!pr.parse(new URL(def_location + url), new URL(def_location + url_s))){
			    return null;
			}
		} catch (ParserConfigurationException | SAXException | IOException e) { 
			return null;
			//e.printStackTrace();
		}
        NodeList constrList = pr.getQoSConstraints(pr.getQoSProfiles().item(0));		//parsing first (default) profile
        for(int i=0;i<constrList.getLength();i++){
        	org.w3c.dom.Node constr = (org.w3c.dom.Node) constrList.item(0);
        	if(pr.getQoSAttribute(constr).equals(attrName)){
        		retValue = pr.getQoSValue(constr);
        	}
        	else{
        		continue;
        	}
        }
        return retValue;
	}

}
