/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package wssl.parser;

import com.parctechnologies.eclipse.Atom;
import com.parctechnologies.eclipse.CompoundTerm;
import com.parctechnologies.eclipse.CompoundTermImpl;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardOpenOption;
import java.util.ArrayList;
import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;
import org.apache.jena.iri.IRI;
import org.apache.jena.iri.impl.IRIFactoryImpl;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;



/**
 * Implements a WSSL parser using DOM
 * @author George Baryannis
 */
public class DOMParser2 implements Parser{
    
    private Document doc;
    private String action;
    private ArrayList<String> variables = new ArrayList<>();
    
    @Override
    public boolean parse(URL wsslfile, URL schemafile) throws ParserConfigurationException, SAXException, IOException {
        SchemaFactory xsFact = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
        Schema schema = xsFact.newSchema(schemafile);
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        dbf.setNamespaceAware(true);
        dbf.setIgnoringElementContentWhitespace(true);
        dbf.setSchema(schema);
        DocumentBuilder db; 
        doc = null;
        db = dbf.newDocumentBuilder();
        doc = db.parse(wsslfile.toString());
        Validator validator = schema.newValidator();
        validator.validate(new DOMSource(doc));
        return true;
    }
    
    @Override
    public NodeList getInp() {
        //get a nodelist of input elements
        return doc.getElementsByTagName("input");
    }
    
    @Override
    public NodeList getOutp() {
        //get a nodelist of output elements
        return doc.getElementsByTagName("output");
    }     

    @Override
    public NodeList getPre() {
        //get a nodelist of precondition elements
        return doc.getElementsByTagName("precondition");
    }

    @Override
    public NodeList getPost() {
        //get a nodelist of postcondition elements
        return doc.getElementsByTagName("postcondition");
    }
    
    @Override
    public NodeList getCR() {
        //get a nodelist of causal relationship elements
        return doc.getElementsByTagName("causal");
    }
    
    @Override
    public NodeList getGoals() {
        //get a nodelist of goal elements
        return doc.getElementsByTagName("goal");
    }
    
    @Override
    public NodeList getQoSProfiles() {
        //get a nodelist of QoSProfile elements
        return doc.getElementsByTagName("QoSProfile");
    }

    @Override
    public NodeList getQoSConstraints(Node profileNode) {
        //get a nodelist of constraint elements
        return profileNode.getChildNodes();
    } 
    
    @Override
    public String getQoSOperator(Node constraintNode) {
        return constraintNode.getChildNodes().item(0).getNodeName();
    }

    @Override
    public String getQoSAttribute(Node constraintNode) {
        NodeList nl = constraintNode.getChildNodes().item(0).getChildNodes();
        for(int i=0;i<nl.getLength();i++){
            Element curNode = (Element) nl.item(i);
            String type = curNode.getAttribute("type");
            if(type.startsWith("xs")){
                continue;
            }
            else{
                return type;
            }
        }
        return null;
    }

    @Override
    public float getQoSValue(Node constraintNode) {
        NodeList nl = constraintNode.getChildNodes().item(0).getChildNodes();
        for(int i=0;i<nl.getLength();i++){
            Element curNode = (Element) nl.item(i);
            String type = curNode.getAttribute("type");
            if(type.startsWith("xs")){
                return new Float(curNode.getTextContent()).floatValue();
            }
            else{
                continue;
            }
        }        
        return 0;
    }    
    
    @Override
    public Node getService() {
        //get a node corresponding to the service element
        return doc.getElementsByTagName("service").item(0);
    }
    
    @Override
    public void writePre(NodeList inpList, NodeList preList, int numOut, Path path)  throws InvalidFormatException, IOException{             //Assuming action contains only primitive terms as arguments
        String writeOut = "";
        String[] inputs = new String[inpList.getLength()];
        try{
            for(int i=0;i<inpList.getLength();i++){      //writing inputs
                Element inp = (Element) inpList.item(i);
                inputs[i] = NameToProlog(inp.getAttribute("name"), 'p');
            }
            Element service = (Element) getService();
            action = NameToProlog(service.getAttribute("name"), 'a') + "(";
            for(int i=0;i<preList.getLength();i++){                    
                Node equiv_r = preList.item(i).getFirstChild(); //Node containing precondition (as a logical expression)
                Element equiv_rn = (Element) equiv_r;
                NodeList preds = equiv_rn.getElementsByTagName("predicate");
                boolean hasAcc = false;
                for(int j=0;j<preds.getLength();j++){       //searching for accident clauses
                    Element pred = (Element) preds.item(j);
                    if(pred.getAttribute("name").equalsIgnoreCase("acc")){
                        hasAcc = true;
                    }
                }
                if(hasAcc){                                         //Accident-specific preconditions are NOT supported (unrelated to services)
                    if(!equiv_r.getNodeName().equals("implies")){
                        throw new InvalidFormatException("acc");
                    }
                    else{
                        equiv_r = equiv_r.getFirstChild().getNextSibling(); //Node containing Pi(z)
                    }

                }
                if(i==0){           //finding name of input state variable to create head of action precondition
                    Element pre = (Element) preList.item(i);
                    preds = pre.getElementsByTagName("holds");
                    if(preds.getLength()==0){
                        throw new InvalidFormatException("holds");
                    }
                    pre = (Element) preds.item(0);
                    String in_state = NameToProlog(pre.getAttribute("state"), 'v');
                    for(int j=0;j<inputs.length;j++){
                        writeOut += "holds(hasinput(" + inputs[j] + "), " + in_state + "), ";
                        action += inputs[j] + ", ";
                    }
                    for(int j=0;j<numOut;j++){
                        action += "_, ";
                    }                    
                    action = action.substring(0,action.length()-2) + ")";
                    writeOut = "poss(" + action + ", Z_IN) :- " + writeOut;
                }
                writeOut += LogExpToProlog(equiv_r) + ", ";        
            }
            writeOut = writeOut.substring(0,writeOut.length()-2) + ".\n";
            try (OutputStream out = new BufferedOutputStream(Files.newOutputStream(path, StandardOpenOption.CREATE, StandardOpenOption.APPEND))) {
                //System.out.println(writeOut);
                byte data[] = writeOut.getBytes();
                out.write(data, 0, data.length);
                out.flush();
                out.close();
            }
        } catch (InvalidFormatException | IOException ex) {
            throw ex;
        }
    }
    
    @Override
    public void writePost(NodeList outpList, NodeList postList, Path path)  throws InvalidFormatException, IOException{             
        String writeOut = "", clauseAb = "";
        String[] outputs = new String[outpList.getLength()];
        try{
        for(int i=0;i<outpList.getLength();i++){      //writing outputs
                Element outp = (Element) outpList.item(i);
                outputs[i] = NameToProlog(outp.getAttribute("name"), 'p');
                action = action.replaceFirst("_", outputs[i]);
        }
        String glob_state_update[] = {null, "", "", null};      //{state_before, theta_+, theta_-, state_after}
        String glob_delta = null;
        Boolean glob_isRamify = false;
        for(int i=0;i<postList.getLength();i++){
            // rhs of state update axiom (Delta(z) /\ z' = z - theta_- + theta_+
            Node rhs = postList.item(i).getFirstChild();
            ArrayList<Node> disjuncts = new ArrayList<>();
            if(rhs.getNodeName().equals("or")){         //disjunctive state update axiom
                NodeList orList = rhs.getChildNodes();
                for(int k=0;k<orList.getLength();k++){
                    disjuncts.add(orList.item(k)); 
               }
            }
            else {
                disjuncts.add(rhs);     //simple state update axiom (only one disjunct)
            }
            String state_update[] = {null, null, null, null};
            String delta = null;
            for(int k=0;k<disjuncts.size();k++){
                Node disjunct = (Node) disjuncts.get(k);
                if(!disjunct.getNodeName().equals("exists")){
                    throw new InvalidFormatException("exists");
                }
                Node rhs_child = disjunct.getChildNodes().item(1);
                boolean isNormal, isRamify=false;
                switch (rhs_child.getNodeName()) {
                    case "and": {   //rhs contains Delta(z)
                        String res[] = StateRHSToProlog(rhs_child);
                        System.arraycopy(res, 0, state_update, 0, 4);
                        delta = res[4];
                        isRamify = Boolean.parseBoolean(res[5]);
                        isNormal = Boolean.parseBoolean(res[6]);
                        if(isRamify){
                            if(i>0){
                                throw new InvalidFormatException("ramify2");
                            }
                        }
                        else{
                            if(i>0&&glob_isRamify==true){
                                throw new InvalidFormatException("ramify2");
                            }                                
                        }
                        /*String states[] = getStates(state_update[0]);
                        state_update[1] = states[0];
                        state_update[2] = states[1];*/
                        break;
                    }
                    case "equal": { //rhs contains only state update
                        if(i>0&&glob_isRamify==true){
                            throw new InvalidFormatException("ramify2");
                        }                            
                        state_update = StateUpdateToProlog(rhs_child);
                        isNormal = true;

                        break;
                    }
                    case "ramify": { //rhs is a ramify clause
                        if(i>0){
                            throw new InvalidFormatException("ramify2");
                        }
                        state_update = StateUpdateToProlog(rhs_child);
                        isNormal = true;
                        isRamify = true;
                        break;                            
                    }    
                    default:
                        //System.out.println(rhs_child.getNodeName());
                        throw new InvalidFormatException("no-rhs");
                }
                if(i==0){       //the type (update/ramify) of the first postcondition defines the global type
                    glob_isRamify = isRamify;
                }
                if(delta!=null){
                    if(glob_delta!=null){
                        glob_delta += ", " + delta;
                    }
                    else{
                        glob_delta = delta;
                    }
                }
                if(isNormal){
                    if(i==0){        //normal (non-accident) disjunct of first postcondition yields state names and adds to positive/negative clauses
                        System.arraycopy(state_update, 0, glob_state_update, 0, 4);
                    }
                    else{
                        if(!glob_state_update[1].equals("")){
                            glob_state_update[1] += ", " + state_update[1];     //add theta_+
                        }
                        //System.out.println("*"+glob_state_update[2]+"*");
                        if(!glob_state_update[2].equals("")){
                            glob_state_update[2] += ", " + state_update[2];     //add theta_-
                        }
                    }
                }
                else{
                    if(!clauseAb.equals("")){
                        clauseAb += ";\n";
                    }
                    if(delta!=null){
                        clauseAb += delta + ", ";
                    }
                    if(isRamify){
                        clauseAb += "ramify(";
                    }
                    else{
                        clauseAb += "update(";
                    }
                     clauseAb += state_update[0] + ", [" + state_update[1] + "], [" + state_update[2] + "], " + state_update[3] + ")";
                }
            }
        }
        writeOut += "state_update(" + glob_state_update[0] + ", " + action + ", " + glob_state_update[3] + ") :- ";
        if(glob_delta!=null){
            writeOut += glob_delta + ", ";
        }
        if(glob_isRamify){
            writeOut += "ramify(";
        }
        else{
            writeOut += "update(";
        }
        writeOut += glob_state_update[0] + ", [" + glob_state_update[1];
        if(!glob_state_update[1].equals("")){
            writeOut += ", ";
        }
        for(int i=0;i<outputs.length;i++){
                writeOut += "hasoutput(" + outputs[i] + "), ";
        }
        writeOut = writeOut.substring(0,writeOut.length()-2);
        writeOut += "], [" + glob_state_update[2] + "], " + glob_state_update[3] + ").\n";
        if(!clauseAb.equals("")){
            writeOut += "ab_state_update(" + glob_state_update[0] + ", " + action + ", " + glob_state_update[3] + ") :- " + clauseAb + ".\n";
        }
        try (OutputStream out = new BufferedOutputStream(Files.newOutputStream(path, StandardOpenOption.CREATE, StandardOpenOption.APPEND))) {
            //System.out.println(writeOut);            
            byte data[] = writeOut.getBytes();
            out.write(data, 0, data.length);
            //data = "init(Z_IN) :- Z_IN = [hasinput(PAYFORM), hasinput(USER), loggedin(USER)].\n".getBytes();
            //out.write(data, 0, data.length);
            out.flush();
        }            
        } catch (InvalidFormatException | IOException ex) {
            throw ex;
        }
    } 

    @Override
    public void writeCR(NodeList crList, Path path) throws InvalidFormatException, IOException {
        String writeOut = "";
        try{
        for(int i=0;i<crList.getLength();i++){
            checkCR(crList.item(i));
            Node gamma = crList.item(i).getFirstChild().getFirstChild();
            Node causes = gamma.getNextSibling();
            writeOut += LogExpToProlog(causes) + " :- " + LogExpToProlog(gamma) + ".\n";       
        }
        try (OutputStream out = new BufferedOutputStream(Files.newOutputStream(path, StandardOpenOption.CREATE, StandardOpenOption.APPEND))) {
            byte data[] = writeOut.getBytes();
            out.write(data, 0, data.length);
            out.flush();
        }
        } catch (InvalidFormatException | IOException ex) {
            throw ex;
        }
        
    }
    
    @Override
    public String writeGoals(NodeList goalList) throws InvalidFormatException{
        String goals = "";
        for(int i=0;i<goalList.getLength();i++){
            if(i>0){
                goals += ", ";
            }
            goals += LogExpToProlog(goalList.item(i).getFirstChild());
        }
        return (goals + ".");
    }
    
    @Override
    public CompoundTerm[] writeGoalsCT(NodeList goalList) throws InvalidFormatException {
        //goalstr = goalstr.substring(0, goalstr.length()-1);
        //String[] goals = goalstr.split(", ");
        CompoundTerm[] compterms = new CompoundTerm[goalList.getLength()];
        for(int i=0;i<goalList.getLength();i++){
            compterms[i] = LogExpToCompoundTerm(goalList.item(i).getFirstChild());
        }
        return compterms;
    }
    
        /**
     * Returns the FLEX (sub-)clause that corresponds to the logical expression.
     * contained in node le
     * @param le node containing a WSSL logical expression
     * @return {@link CompoundTerm} containing the FLEX clause
     * @throws InvalidFormatException
     */
    public CompoundTerm LogExpToCompoundTerm(Node le) throws InvalidFormatException{
        try{
            if(le==null){
                throw new InvalidFormatException("le");
            }
            String type = le.getNodeName();
            switch (type){
                case "true": return new Atom("true");
                case "false": return new Atom("false");    
                case "predicate": {
                    Element predicate = (Element) le;
                    String name = predicate.getAttribute("name");
                    if(!name.equalsIgnoreCase("poss") && !name.equalsIgnoreCase("causes") && !name.equalsIgnoreCase("acc") && !name.equalsIgnoreCase("init") && !name.equalsIgnoreCase("state_update")){
                        name = NameToProlog(name, 'a');         
                    }
                    NodeList terms = predicate.getChildNodes();
                    int children = terms.getLength();
                    if(name.equalsIgnoreCase("causes")){
                        children--;
                    }
                    CompoundTerm[] compterms = new CompoundTerm[terms.getLength()];
                    for(int i=0;i<children;i++){                   
                        compterms[i] = TermToCompoundTerm(terms.item(i));
                    }
                    return new CompoundTermImpl(name, compterms);
                }
                case "holds": {
                    Element holds = (Element) le;
                    variables.add(NameToProlog(holds.getAttribute("state"), 'v'));
                    //return new CompoundTermImpl("holds", TermToProlog(le.getFirstChild()), new Atom(null));
                    return new CompoundTermImpl("holds", TermToProlog(le.getFirstChild()), null);
                }    
                case "and": {
                    NodeList operands = le.getChildNodes();
                    CompoundTerm[] compterms = new CompoundTerm[operands.getLength()];
                    for(int i=0;i<operands.getLength();i++){
                        compterms[i] = LogExpToCompoundTerm(operands.item(i));
                    }
                    return new CompoundTermImpl(",", compterms);
                }
                case "or": {
                    NodeList operands = le.getChildNodes();
                    CompoundTerm[] compterms = new CompoundTerm[operands.getLength()];
                    for(int i=0;i<operands.getLength();i++){
                        compterms[i] = LogExpToCompoundTerm(operands.item(i));
                    }
                    return new CompoundTermImpl(";", compterms);
                }                    

                default: return null;

            }
        } catch (InvalidFormatException ex) {         
            throw ex;
        }
                
    }
    
/**
     * Returns the FLEX (sub-)clause that corresponds to the term contained
     * in node le.
     * @param le node containing a WSSL term
     * @return {@link CompoundTerm} containing the FLEX clause
     * @throws InvalidFormatException
     */
    public CompoundTerm TermToCompoundTerm(Node le) throws InvalidFormatException{
        try{
            if(le==null){
                throw new InvalidFormatException("te");
            }        
            String name;
            Element term = (Element) le;
            if(term.getNodeName().equals("var")){
                if(term.hasAttribute("value")){                             //variable is instantiated
                    //return new CompoundTermImpl ("", new Atom(NameToProlog(term.getAttribute("value"),'l')));
                    return new Atom(NameToProlog(term.getAttribute("value"),'l'));
                }
                else if(term.hasAttribute("name")){                         //we only know the variable name
                    variables.add(NameToProlog(term.getAttribute("name"),'v'));
                    return null;
                    //return new Atom(null);
                }
                else{
                    throw new InvalidFormatException("var");
                }
            }
            String type = term.getAttribute("type");
            if(type.startsWith("xs:")){                //supports elementary data values, while constructed ones need a bit more type-specific code
                return new Atom(le.getFirstChild().getTextContent());
            }
            else if(type.equals("HasInput") || type.equals("HasOutput")){
                name = type.toLowerCase() + "(";
                Element varterm = (Element) le;
                NodeList terms = varterm.getChildNodes();
                CompoundTerm[] compterms = new CompoundTerm[terms.getLength()];
                for(int i=0;i<terms.getLength();i++){                
                    compterms[i] = TermToCompoundTerm(terms.item(i));
                }
                return new CompoundTermImpl(name, compterms);
                
            }
            else{       
                name = NameToProlog(term.getAttribute("type"), 'a');
                Element varterm = (Element) le;
                NodeList terms = varterm.getChildNodes();
                
                int j=0;
                for(int i=0;i<terms.getLength();i++){
                    if(terms.item(i).getNodeName().startsWith("#")){
                        continue;
                    }
                    j++;
                }
                CompoundTerm[] compterms = new CompoundTerm[j];
                j=0;
                for(int i=0;i<terms.getLength();i++){
                    if(terms.item(i).getNodeName().startsWith("#")){
                        continue;
                    }              
                    compterms[j++] = TermToCompoundTerm(terms.item(i));
                }
                return new CompoundTermImpl(name, compterms);
            }
        } catch (InvalidFormatException ex) {         
            throw ex;
        }
    }
    
    /**
     * Returns the FLEX (sub-)clause that corresponds to the logical expression.
     * contained in node le
     * @param le node containing a WSSL logical expression
     * @return String containing the FLEX clause
     * @throws InvalidFormatException
     */
    public String LogExpToProlog(Node le) throws InvalidFormatException{
        try{
            if(le==null){
                throw new InvalidFormatException("le");
            }
            String writeOut = "";
            String type = le.getNodeName();
            switch (type){
                case "true": return "true";
                case "false": return "false";    
                case "predicate": {
                    Element predicate = (Element) le;
                    String name = predicate.getAttribute("name");
                    if(name.equalsIgnoreCase("poss") || name.equalsIgnoreCase("causes") || name.equalsIgnoreCase("acc") || name.equalsIgnoreCase("init") || name.equalsIgnoreCase("state_update")){
                        writeOut += name + "(";         //predefined predicate
                    }
                    else{
                        writeOut += NameToProlog(name, 'a') + "(";
                    }
                    NodeList terms = predicate.getChildNodes();
                    int children = terms.getLength();
                    if(name.equalsIgnoreCase("causes")){
                        children--;
                    }
                    for(int i=0;i<children;i++){
                        if(i>0){
                            writeOut += ", ";
                        }                    
                        writeOut += TermToProlog(terms.item(i));
                    }
                    return (writeOut + ")");
                }
                case "holds": {
                    Element holds = (Element) le;
                    /*Element predicate = (Element) le.getFirstChild();
                    writeOut += "holds(" + NameToProlog(predicate.getAttribute("name"), 'a') + "(";
                    NodeList terms = predicate.getChildNodes();
                    for(int i=0;i<terms.getLength();i++){
                        if(i>0){
                            writeOut += ", ";
                        }                     
                        try {
                            writeOut += TermToProlog(terms.item(i));
                        } catch (InvalidFormatException ex) {
                            throw ex;
                        }
                    }*/
                    writeOut += "holds(" + TermToProlog(le.getFirstChild());

                    return (writeOut + ", " + NameToProlog(holds.getAttribute("state"), 'v') + ")");
                }    
                case "not": {
                    return ("not("+ LogExpToProlog(le.getFirstChild()) +")");
                }
                case "and": {
                    NodeList operands = le.getChildNodes();
                    String ret = "";
                    for(int i=0;i<operands.getLength();i++){
                        if(i>0){
                            ret += ", ";
                        }
                        ret += LogExpToProlog(operands.item(i));
                    }
                    return ret;
                }
                case "or": {
                    NodeList operands = le.getChildNodes();
                    String ret = "";
                    for(int i=0;i<operands.getLength();i++){
                        if(i>0){
                            ret += "; ";
                        }
                        ret += LogExpToProlog(operands.item(i));
                    }
                    return ret;
                }
                case "implies": {
                    Node left = le.getFirstChild();
                    return (LogExpToProlog(left) + "-> " + LogExpToProlog(left.getNextSibling()));
                }
                case "equivalent": {
                    Node left_n = le.getFirstChild();
                    String left = LogExpToProlog(left_n);
                    String right = LogExpToProlog(left_n.getNextSibling());
                    return (left + "-> " + right + ", " + right + "-> " + left);
                }
                case "equal": {
                    Node left = le.getFirstChild();
                    Element term = (Element) left;
                    String tp = term.getAttribute("type");
                    if(tp.equals("State")){       //it's a goal
                        return (TermToProlog(left));
                    }
                    return (TermToProlog(left) + " is " + TermToProlog(left.getNextSibling()));
                }
                default: return null;

            }
        } catch (InvalidFormatException ex) {         
            throw ex;
        }
                
    }    
    
    /**
     * Returns the FLEX (sub-)clause that corresponds to the term contained
     * in node le.
     * @param le node containing a WSSL term
     * @return String containing the FLEX clause
     * @throws InvalidFormatException
     */
    public String TermToProlog(Node le) throws InvalidFormatException{
        try{
            if(le==null){
                throw new InvalidFormatException("te");
            }        
            String writeOut = "";
            Element term = (Element) le;
            if(term.getNodeName().equals("var")){
                if(term.hasAttribute("value")){                             //variable is instantiated
                    return (NameToProlog(term.getAttribute("value"),'l'));
                }
                else if(term.hasAttribute("name")){                         //we only know the variable name
                    return (NameToProlog(term.getAttribute("name"),'v'));
                }
                else{
                    throw new InvalidFormatException("var");
                }
            }
            String type = term.getAttribute("type");
            if(term.getNodeName().equals("empty")){
                return writeOut;
            }
            else if(type.equals("State")){            //goal translation (only one goal type supported for now)
                Element varterm = (Element) le;
                NodeList terms = varterm.getElementsByTagName("term");                
                writeOut = "state_update(" + TermToProlog(terms.item(4))
                                    + ", " + TermToProlog(terms.item(1))
                                    + ", " + TermToProlog(term.getNextSibling()) + ")";
            }
            else if(term.getNodeName().equals("minus")){
                throw new InvalidFormatException("minus");
            }
            else if(term.getNodeName().equals("plus")){
                Element left = (Element) le.getFirstChild();
                if(left.getNodeName().equals("var")){
                    writeOut += "[" + TermToProlog(left.getNextSibling()) + "|" + TermToProlog(left) + "]";
                }
                else{
                    writeOut += "[" + TermToProlog(left) + "|" + TermToProlog(left.getNextSibling()) + "]";
                } 
            }
            else if(term.getNodeName().equals("circ")){
                NodeList fluents = le.getChildNodes();
                for(int i=0;i<fluents.getLength();i++){
                    if(i>0){
                        writeOut += ", ";
                    }                    
                    writeOut += TermToProlog(fluents.item(i));
                }
                return (writeOut);
            }
            else if(type.startsWith("xs:")){                //supports elementary data values, while constructed ones need a bit more type-specific code
                return le.getFirstChild().getTextContent();
            }
            /*else if(type.equals("variable")){
                Element varterm = (Element) le;
                Element var = (Element) varterm.getElementsByTagName("var").item(0);
                return NameToProlog(var.getAttribute("name"), 'v');
            }*/
            else if(type.equals("HasInput") || type.equals("HasOutput")){
                writeOut += type.toLowerCase() + "(";
                Element varterm = (Element) le;
                NodeList terms = varterm.getChildNodes();
                for(int i=0;i<terms.getLength();i++){
                    if(i>0){
                        writeOut += ", ";
                    }                 
                    writeOut += TermToProlog(terms.item(i));
                }
                writeOut += ")";            
            }
            else{       
                writeOut += NameToProlog(term.getAttribute("type"), 'a') + "(";
                Element varterm = (Element) le;
                NodeList terms = varterm.getChildNodes();
                for(int i=0;i<terms.getLength();i++){
                    if(terms.item(i).getNodeName().startsWith("#")){
                        continue;
                    }              
                    writeOut += TermToProlog(terms.item(i)) + ", ";
                }
                writeOut = writeOut.substring(0,writeOut.length()-2) + ")";
            }
            return writeOut;
        } catch (InvalidFormatException ex) {         
            throw ex;
        }
    }
    
    /**
     * Returns the FLEX (sub-)clause that corresponds to the right-hand side of
     * the state update axiom, contained in node le.
     * @param le node containing the right-hand size of a WSSL state update axiom
     * @return String array as follows:
     *      [0] = clause corresponding to the "before" state in a state update axiom
     *      [1] = clause corresponding to the "theta+" part of a state update axiom
     *      [2] = clause corresponding to the "theta-" part of a state update axiom
     *      [3] = clause corresponding to the "after" state in a state update axiom 
     *      [4] = clause corresponding to the Delta function in a state update axiom 
     *      [5] = true if the parsed axiom is a ramify clause, false otherwise
     *      [6] = false if the parsed axiom is an accident case, true otherwise
     * @throws InvalidFormatException
     */
    public String[] StateRHSToProlog(Node le) throws InvalidFormatException{
        NodeList operands = le.getChildNodes();
        String ret[] = {"", "", "", "", null, "false", "false"};
        boolean parsedStates = false;
        try{
            for(int i=0;i<operands.getLength();i++){
                String name = operands.item(i).getNodeName();
                //System.out.println(i + name);
                if(name.equals("equal") || name.equals("ramify")){ //state update or ramify clause
                    String su_ret[] = StateUpdateToProlog(operands.item(i));
                    if(!parsedStates){
                        System.arraycopy(su_ret, 0, ret, 0, 4);
                        parsedStates = true;
                    }
                    else{
                        ret[1] += ", " + su_ret[1];
                        ret[2] += ", " + su_ret[2];         
                    }
                }
                else{       //searching for accident cases
                    Element operand = (Element) operands.item(i);
                    NodeList preds = operand.getElementsByTagName("predicate");
                    boolean hasAcc = false;
                    /*if(operand.getAttribute("name").equalsIgnoreCase("acc")){
                        hasAcc = true;
                    }
                    else{*/
                    for(int j=0;j<preds.getLength();j++){
                        Element pred = (Element) preds.item(j);
                        if(pred.getAttribute("name").equalsIgnoreCase("acc")){
                            hasAcc = true;
                        }
                    }    
                    //}
                    if(!hasAcc){     //no accident found, so it must be a normal Delta(z)
                        /*if(!isFirst){
                            ret[0] += ", ";
                            isFirst = false;
                        }*/
                        if(ret[4]==null){
                            ret[4] = LogExpToProlog(operands.item(i));
                        }
                        else{
                            ret[4] += LogExpToProlog(operands.item(i));
                        }
                        ret[6] = "true";
                    }
                    else{        //contains accident predicate
                        if(operand.getNodeName().equals("forall") &&        //check if operand is forall(c) not(Acc(c,s)))
                           operand.getFirstChild().getNextSibling().getNodeName().equals("not") &&
                           operand.getFirstChild().getNextSibling().getFirstChild().getNodeName().equals("predicate")){
                           Element acc = (Element) operand.getFirstChild().getNextSibling().getFirstChild();
                           if(acc.getAttribute("name").equalsIgnoreCase("acc")){
                               ret[6] = "true";
                           }
                           else{
                               ret[6] = "false";
                           }
                        }
                        else{
                            ret[6] = "false";
                        }
                    }    
                }
            }
            /*if(isValid==false){
                throw new InvalidFormatException("no-rhs");
            }*/
            return ret;
        } catch (InvalidFormatException ex) {         
            throw ex;
        }
    }    
    
    /**
     * Returns the FLEX clause that corresponds to the state update axiom 
     * contained in node le.
     * @param su node containing a WSSL state update axiom
     * @return String array as follows:
     *      [0] = clause corresponding to the "before" state in a state update axiom
     *      [1] = clause corresponding to the "theta+" part of a state update axiom
     *      [2] = clause corresponding to the "theta-" part of a state update axiom
     *      [3] = clause corresponding to the "after" state in a state update axiom 
     *      [4] = true if the parsed axiom is a ramify clause, false otherwise
     * @throws InvalidFormatException
     */    
    public String[] StateUpdateToProlog(Node su) throws InvalidFormatException{
        String ret[] = {"", "", "", "", "false"};
        try{
            switch (su.getNodeName()) {
                case "ramify":
                    //Contains ramify macro
                    ret[4] = "true";
                    NodeList terms = su.getChildNodes();
                    /*for(int j=0;j<terms.getLength();j=j+3){
                        Element var = (Element) terms.item(j);
                        NodeList var_c =  var.getElementsByTagName("var");
                        if(var_c==null){
                            throw new InvalidFormatException("ramify");
                        }
                        else{
                            var = (Element) var_c.item(0);
                            if(j==0){
                                ret[0] = NameToProlog(var.getAttribute("name"), 'v');
                            }
                            else{
                                ret[3] = NameToProlog(var.getAttribute("name"), 'v');
                            }                                    
                        }
                    }*/
                    for(int j=0;j<terms.getLength();j++){
                        ret[j] = TermToProlog(terms.item(j));
                    }
                    break;
                case "equal":
                    Node state_formula = null;
                    Node update_formula = null;
                    Node update_left = su.getFirstChild();
                    Node update_right = update_left.getNextSibling();
                    if(update_right.getNodeName().equals("minus") || update_right.getNodeName().equals("plus")){
                        update_formula = update_right;
                        state_formula = update_left;
                    }
                    else if(update_left.getNodeName().equals("minus") || update_left.getNodeName().equals("plus")){
                        update_formula = update_left;
                        state_formula = update_right;
                    }
                    else if(update_left.getNodeName().equals("var") && update_right.getNodeName().equals("var")){ //z' = z
                        update_formula = null;
                        state_formula = update_left;
                    }
                    else{
                        throw new InvalidFormatException("wrong-rhs");     //rhs doesn't have a valid state update formula
                    }
                    if(!state_formula.getNodeName().equals("var")){
                        throw new InvalidFormatException("wrong-rhs");
                    }
                    else{
                        Element state = (Element) state_formula;
                        ret[3] = NameToProlog(state.getAttribute("name"), 'v');
                        if(update_formula==null){
                            ret[1] = "";
                            ret[2] = "";
                        }
                        else{
                            switch (update_formula.getNodeName()) {
                                case "minus": {
                                    Element minuend = (Element) update_formula.getFirstChild();
                                    switch (minuend.getNodeName()) {
                                        case "var":     //= State(z) - th_min
                                            ret[0] = NameToProlog(minuend.getAttribute("name"), 'v');
                                            ret[2] = TermToProlog(minuend.getNextSibling());
                                            break;
                                        case "plus":  
                                            Element term = (Element) minuend.getFirstChild();
                                            if(term.getNodeName().equals("var")){   //= (State(z) + th_plus) - th_min
                                                ret[0] = NameToProlog(term.getAttribute("name"), 'v');
                                                ret[1] = TermToProlog(term.getNextSibling());
                                                ret[2] = TermToProlog(minuend.getNextSibling());
                                            }
                                            else{                                                   //= (th_plus + State(z)) - th_min 
                                                ret[1] = TermToProlog(term);
                                                term = (Element) term.getNextSibling();
                                                ret[0] = NameToProlog(term.getAttribute("name"), 'v');
                                                ret[2] = TermToProlog(minuend.getNextSibling());
                                            }
                                            break;
                                        default:
                                            throw new InvalidFormatException("wrong-rhs");
                                    }
                                    break;    
                                }  
                                case "plus": {
                                    Element term = (Element) update_formula.getFirstChild();
                                    switch (term.getNodeName()) {
                                        case "var":
                                            //= State(z) + th_plus
                                            ret[0] = NameToProlog(term.getAttribute("name"), 'v');
                                            ret[1] = TermToProlog(term.getNextSibling());
                                            break;
                                        case "term":
                                            ret[1] = TermToProlog(term);
                                            term = (Element) term.getNextSibling();
                                            switch (term.getNodeName()) {
                                                case "var":
                                                    //= th_plus + State(z)
                                                    ret[0] = NameToProlog(term.getAttribute("name"), 'v');
                                                    break;
                                                case "minus":
                                                    //= th_plus + State(z) - th_minus
                                                    term = (Element) term.getFirstChild();
                                                    ret[0] = NameToProlog(term.getAttribute("name"), 'v');
                                                    ret[2] = TermToProlog(term.getNextSibling());
                                                    break;
                                                default:
                                                    throw new InvalidFormatException("wrong-rhs");
                                            }
                                            break;
                                        case "minus":  
                                            Element minuend = (Element) term.getFirstChild();
                                            if(minuend.getNodeName().equals("var")){   //= (State(z) - th_min) + th_plus
                                                ret[0] = NameToProlog(minuend.getAttribute("name"), 'v');
                                                ret[1] = TermToProlog(term.getNextSibling());
                                                ret[2] = TermToProlog(minuend.getNextSibling());
                                            }
                                            else{                                                   
                                                throw new InvalidFormatException("wrong-rhs");
                                            }
                                            break;
                                        default:                                                    
                                            throw new InvalidFormatException("wrong-rhs");
                                    }
                                    break;    
                                }
                                default:
                                    throw new InvalidFormatException("wrong-rhs"); //assuming that an action has at least one effect (positive or negative)
                            }
                        }
                    }
                    break;
                default:
                    throw new InvalidFormatException("wrong-rhs");  //state update is neither an equality nor a ramify clause
            }
        } catch (InvalidFormatException ex) {         
            throw ex;
        }        
        return ret;
    }

    /**
     * Checks the format of a WSSL causal relationship
     * @param cr a {@link Node} containing a WSSL causal relationship
     * @throws InvalidFormatException
     */    
    public void checkCR(Node cr) throws InvalidFormatException{
        if(cr.getFirstChild()==null || cr.getFirstChild().getFirstChild()==null || cr.getFirstChild().getFirstChild().getNextSibling()==null && !cr.getFirstChild().getFirstChild().getNextSibling().getNodeName().equals("predicate")){
            throw new InvalidFormatException("pred-cr");
        }
        else{
            Element pred = (Element) cr.getFirstChild().getFirstChild().getNextSibling();
            if(!pred.getAttribute("name").equalsIgnoreCase("causes")){
                throw new InvalidFormatException("pred-cr");
            }
            else if(pred.getChildNodes().getLength()!=6){
                throw new InvalidFormatException("sixpred-cr");
            }
        }
    }    
    
    /**
     * Translate a WSSL action or variable name to FLEX
     * @param name the name to be translated
     * @param mode 'a' if the name to be translated corresponds to an action,
     *             'v' if it corresponds to a variable
     * @return String containing the FLEX translation
     * @throws InvalidFormatException
     */
    public String NameToProlog(String name, char mode) throws InvalidFormatException{
            if(name==null || name.equals("")) {
                throw new InvalidFormatException("iriv");
            }
            if(mode=='a' || mode=='p'){ //action or action parameter

                    IRIFactoryImpl IRIFactory = new IRIFactoryImpl();
                    IRI nameIRI = IRIFactory.create(name);
                    if(nameIRI.hasViolation(false)){
                        throw new InvalidFormatException("iri");
                    }
                    if(nameIRI.getRawFragment()==null) {
                        throw new InvalidFormatException("irif");
                    }
                    if(mode=='a'){
                        return nameIRI.getRawFragment().toLowerCase();   
                    }
                    else{
                        return nameIRI.getRawFragment().toUpperCase();
                    }
                    
                }
            else if(mode=='v'){
                return name.substring(1).toUpperCase();   //variable
            }
            else{
                return name;
            }
    }
    
    /**
     * Returns the String at the given position of the variables array
     * @param position index of the String to return
     * @return String value at the particular position of the array
     */
    public String getVariable(int position){
        return variables.get(position);
    }
    
    /**
     * Returns the size of the variables array
     * @return integer containing the number of variable names in the array
     */
    public int getSize(){
        return variables.size();
    }    
    
}

    

