/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package wssl.parser;

/**
 * Extends RuntimeException with reasons specific to WSSL.
 * @author George Baryannis
 * 
 */
@SuppressWarnings("serial")
public class InvalidFormatException extends RuntimeException {
    /**
     * Contains an abbreviation of the cause of the exception
     */
    private String reason;
    /**
     * Returns the cause of the exception (as an abbreviation).
     * @param reason abbreviation of the cause of the exception
     */
    public InvalidFormatException(String reason){
        this.reason = reason;
    }
    
    /**
     * Associates reason with a WSSL-specific textual description of the exception.
     * @return the explanation for the caused exception
     */
    @Override
    public String toString(){
        switch(reason){
            case "inp": return "Input is not a valid input formula. No HasInput fluent found.";
            case "outp": return "Output is not a valid output formula. No HasOutput fluent found.";
            case "pred": return "Precondition isn't a valid action precondition axiom. No poss predicate found.";
            case "pred-post": return "Postcondition isn't a valid state update axiom. No poss predicate found.";
            case "pred-cr": return "Causal relationship is invalid. No causes predicate found.";                
            case "binpred": return "Precondition isn't a valid action precondition axiom. Poss predicate must be binary.";
            case "binpred-post": return "Postcondition isn't a valid state update axiom. Poss predicate must be binary.";
            case "sixpred-cr": return "Postcondition isn't a valid state update axiom. Poss predicate must be binary.";    
            case "exists": return "Postcondition isn't a valid state update axiom. No existential quantification found.";
            case "no-rhs": return "Postcondition isn't a valid state update axiom. No state update found.";    
            case "wrong-rhs": return "Postcondition isn't a valid state update axiom. Invalid state update found.";
            case "minus": return "Subtraction of states can only be part of a state update axiom.";
            case "ramify": return "Postcondition isn't a valid state update axiom. Invalid ramify syntax.";    
            case "ramify2": return "Regular postconditions and ramify clauses cannot co-exist in a service specification.";                
            case "iri": return "IRI syntax violation.";
            case "irif": return "Attempted translation of an IRI without a fragment.";
            case "iriv": return "Attempted translation of an empty IRI or variable name.";
            case "le": return "Attempted translation of an empty logical expression.";
            case "te": return "Attempted translation of an empty term.";
            case "acc": return "Precondition isn't a valid action precondition axiom. Accidents are specified incorrectly.";
            case "action": return "Postconditions refer to different services";
            case "holds": return "Precondition isn't a valid action precondition axiom. No holds predicate found."; 
            case "var": return "Variable tags must have either a name or a value attribute";
            default: return "Invalid format for a precondition or postcondition.";
        }
    }
}
