/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package wssl.validator;

import com.parctechnologies.eclipse.CompoundTerm;
import com.parctechnologies.eclipse.EclipseEngine;
import com.parctechnologies.eclipse.EclipseEngineOptions;
import com.parctechnologies.eclipse.EclipseException;
import com.parctechnologies.eclipse.EmbeddedEclipse;

import java.awt.event.ItemEvent;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.util.LinkedList;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.UIManager;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;

import wssl.parser.DOMParser;
import wssl.parser.InvalidFormatException;

/*import java.util.logging.Level;
import java.util.logging.Logger;*/

/**
 * Simple GUI application to validate a WSSL goal against a WSSL description
 * @author George Baryannis
 */
@SuppressWarnings("serial")
public class ValidatorGUI extends javax.swing.JFrame {

    /**
     * Creates new form ValidatorGUI
     */
    public ValidatorGUI() {
        initComponents();
        this.setTitle("WSSL Validator Tool");
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        goalFormat = new javax.swing.ButtonGroup();
        filePath = new javax.swing.JTextField();
        filePathLabel = new javax.swing.JLabel();
        filePathButton = new javax.swing.JButton();
        schemaPathLabel = new javax.swing.JLabel();
        schemaPath = new javax.swing.JTextField();
        schemaPathButton = new javax.swing.JButton();
        outputPathButton = new javax.swing.JButton();
        outputPath = new javax.swing.JTextField();
        outputPathLabel = new javax.swing.JLabel();
        EclipseToggle = new javax.swing.JCheckBox();
        separator = new javax.swing.JSeparator();
        fluxPathLabel = new javax.swing.JLabel();
        fluxPath = new javax.swing.JTextField();
        fluxPathButton = new javax.swing.JButton();
        Run = new javax.swing.JButton();
        goalPathLabel = new javax.swing.JLabel();
        goalPath = new javax.swing.JTextField();
        goalPathButton = new javax.swing.JButton();
        goalFormatFLUX = new javax.swing.JRadioButton();
        goalFormatWSSL = new javax.swing.JRadioButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setName("WSSL Validator"); // NOI18N

        filePathLabel.setText("WSSL File Path");

        filePathButton.setText("Browse...");
        filePathButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                filePathButtonActionPerformed(evt);
            }
        });

        schemaPathLabel.setText("WSSL Schema Path");

        schemaPath.setName(""); // NOI18N
        schemaPath.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                schemaPathActionPerformed(evt);
            }
        });

        schemaPathButton.setText("Browse...");
        schemaPathButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                schemaPathButtonActionPerformed(evt);
            }
        });

        outputPathButton.setText("Browse...");
        outputPathButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                outputPathButtonActionPerformed(evt);
            }
        });

        outputPath.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                outputPathActionPerformed(evt);
            }
        });

        outputPathLabel.setText("Output Path");

        EclipseToggle.setText("Do not run ECLiPSe");
        EclipseToggle.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                EclipseToggleItemStateChanged(evt);
            }
        });
        EclipseToggle.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                EclipseToggleActionPerformed(evt);
            }
        });

        fluxPathLabel.setText("FLUX Library");

        fluxPath.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fluxPathActionPerformed(evt);
            }
        });

        fluxPathButton.setText("Browse...");
        fluxPathButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fluxPathButtonActionPerformed(evt);
            }
        });

        Run.setText("Run");
        Run.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                RunActionPerformed(evt);
            }
        });

        goalPathLabel.setText("Validation Goal");

        goalPath.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                goalPathActionPerformed(evt);
            }
        });

        goalPathButton.setText("Browse...");
        goalPathButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                goalPathButtonActionPerformed(evt);
            }
        });

        goalFormat.add(goalFormatFLUX);
        goalFormatFLUX.setSelected(true);
        goalFormatFLUX.setText("FLUX Goal");
        goalFormatFLUX.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                goalFormatFLUXActionPerformed(evt);
            }
        });

        goalFormat.add(goalFormatWSSL);
        goalFormatWSSL.setText("WSSL Goal");
        goalFormatWSSL.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                goalFormatWSSLActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGap(17, 17, 17)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addGroup(layout.createSequentialGroup()
                                        .addComponent(filePathLabel)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(filePath, javax.swing.GroupLayout.PREFERRED_SIZE, 188, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(filePathButton))
                                    .addGroup(layout.createSequentialGroup()
                                        .addGap(12, 12, 12)
                                        .addComponent(outputPathLabel)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(outputPath, javax.swing.GroupLayout.PREFERRED_SIZE, 188, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(outputPathButton)))
                                .addGroup(layout.createSequentialGroup()
                                    .addComponent(schemaPathLabel)
                                    .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                    .addComponent(schemaPath, javax.swing.GroupLayout.PREFERRED_SIZE, 188, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                    .addComponent(schemaPathButton))
                                .addComponent(separator, javax.swing.GroupLayout.PREFERRED_SIZE, 363, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addGroup(layout.createSequentialGroup()
                                .addGap(7, 7, 7)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                    .addGroup(layout.createSequentialGroup()
                                        .addComponent(EclipseToggle)
                                        .addGap(245, 245, 245))
                                    .addGroup(layout.createSequentialGroup()
                                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                            .addComponent(fluxPathLabel)
                                            .addComponent(goalPathLabel))
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                            .addGroup(layout.createSequentialGroup()
                                                .addGap(10, 10, 10)
                                                .addComponent(goalFormatFLUX)
                                                .addGap(18, 18, 18)
                                                .addComponent(goalFormatWSSL))
                                            .addGroup(layout.createSequentialGroup()
                                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                                                    .addComponent(goalPath)
                                                    .addComponent(fluxPath, javax.swing.GroupLayout.DEFAULT_SIZE, 188, Short.MAX_VALUE))
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                    .addComponent(fluxPathButton)
                                                    .addComponent(goalPathButton)))))))))
                    .addGroup(layout.createSequentialGroup()
                        .addGap(153, 153, 153)
                        .addComponent(Run, javax.swing.GroupLayout.PREFERRED_SIZE, 86, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(18, Short.MAX_VALUE))
        );

        layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {filePath, fluxPath, outputPath, schemaPath});

        layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {filePathButton, fluxPathButton, outputPathButton, schemaPathButton});

        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(29, 29, 29)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(schemaPath, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(schemaPathLabel)
                    .addComponent(schemaPathButton))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(filePath, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(filePathLabel)
                    .addComponent(filePathButton))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(outputPath, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(outputPathLabel)
                    .addComponent(outputPathButton))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(separator, javax.swing.GroupLayout.PREFERRED_SIZE, 1, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(7, 7, 7)
                .addComponent(EclipseToggle)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(fluxPathButton)
                    .addComponent(fluxPath, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(fluxPathLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(goalPath, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(goalPathLabel)
                    .addComponent(goalPathButton))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(goalFormatFLUX)
                    .addComponent(goalFormatWSSL))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 22, Short.MAX_VALUE)
                .addComponent(Run)
                .addGap(27, 27, 27))
        );

        layout.linkSize(javax.swing.SwingConstants.VERTICAL, new java.awt.Component[] {filePath, fluxPath, outputPath, schemaPath});

        layout.linkSize(javax.swing.SwingConstants.VERTICAL, new java.awt.Component[] {filePathButton, fluxPathButton, outputPathButton, schemaPathButton});

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void schemaPathActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_schemaPathActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_schemaPathActionPerformed

    private void outputPathButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_outputPathButtonActionPerformed
        int returnVal = fc.showOpenDialog(ValidatorGUI.this);
        if (returnVal == JFileChooser.APPROVE_OPTION){
            output = fc.getSelectedFile();
            outputPath.setText(output.getAbsolutePath());
        }
    }//GEN-LAST:event_outputPathButtonActionPerformed

    private void fluxPathActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fluxPathActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_fluxPathActionPerformed

    private void fluxPathButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fluxPathButtonActionPerformed
        int returnVal = fc.showOpenDialog(ValidatorGUI.this);
        if (returnVal == JFileChooser.APPROVE_OPTION){
            flux = fc.getSelectedFile();
            fluxPath.setText(flux.getAbsolutePath());
        }
    }//GEN-LAST:event_fluxPathButtonActionPerformed

    private void RunActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_RunActionPerformed
        URL def_location = Validator.class.getProtectionDomain().getCodeSource().getLocation();
        if(!schemaPath.getText().contains(":")){
            schemaStr = def_location + schemaPath.getText();
        }
        else{
            schemaStr = schemaPath.getText();
        }
        if(!filePath.getText().contains(":")){
            fileStr = def_location + filePath.getText();
        }
        else{
            fileStr = filePath.getText();
        }
        if(!outputPath.getText().contains(":")){
            outputStr = def_location + outputPath.getText();
        }
        else{
            outputStr = outputPath.getText();
        }
        if(!goalPath.getText().contains(":")){
            goalStr = def_location + goalPath.getText();
        }
        else{
            goalStr = goalPath.getText();
        }        
        if(flux==null){
            //System.out.println(def_location + fluxPath.getText());
            //flux = new File(def_location.getFile() + fluxPath.getText());
            flux = new File(fluxPath.getText());
        }
        if(output==null){
            //System.out.println(def_location + outputPath.getText());
            //output = new File(def_location.getFile() + outputPath.getText());
            output = new File(outputPath.getText());
        }        
        
        DOMParser pr = new DOMParser();
        try {           
            if(!pr.parse(new URL("file:/" + fileStr), new URL("file:/" + schemaStr))){
                System.exit(-1);
            }
            URL out_url = new URL("file:/" + outputStr);
            Path out = Paths.get(out_url.toURI());
            pr.writePre(pr.getInp(), pr.getPre(), pr.getOutp().getLength(), out);
            pr.writePost(pr.getOutp(), pr.getPost(), out);
            pr.writeCR(pr.getCR(), out);
            if(EclipseToggle.isSelected()){
                JOptionPane.showMessageDialog(this, "Translation to FLUX completed successfully", "Translation Result",
                                            JOptionPane.INFORMATION_MESSAGE);
            }
            if(!EclipseToggle.isSelected()){
                    // Create some default Eclipse options
                EclipseEngineOptions eclipseEngineOptions = new EclipseEngineOptions();
                //eclipseEngineOptions.setEclipseDir(new File("/C:/Program Files (x86)/ECLiPSe 6.0/"));
                    // Object representing the Eclipse process
                EclipseEngine engine;

                    // Connect the Eclipse's standard streams to the JVM's
                eclipseEngineOptions.setUseQueues(false);

                    // Initialise Eclipse
                engine = EmbeddedEclipse.getInstance(eclipseEngineOptions);
                //System.out.println(flux);
                engine.compile(flux);
                //System.out.println(output);
                engine.compile(output);
                    // Write a message
                
                String query;
                if(goalFormatFLUX.isSelected()){
                    URL url_g = new URL("file:/" + goalStr);
                    Path goal = Paths.get(url_g.toURI());
                    try (InputStream in = new BufferedInputStream(Files.newInputStream(goal, StandardOpenOption.READ))) {
                        byte data[] = new byte[INPUT_SIZE];
                        in.read(data);
                        in.close();
                        query = new String(data);
                    }
                }
                else{
                    DOMParser pr_goal = new DOMParser();
                    if(!pr_goal.parse(new URL("file:/" + goalStr), new URL("file:/" + schemaStr))){
                        System.exit(-1);
                    }
                    query = pr_goal.writeGoals(pr_goal.getGoals());
                    //System.out.println(query);
                }
                CompoundTerm res = engine.rpc(query);
                 
                CompoundTerm goal1 = (CompoundTerm) res.arg(1);
                CompoundTerm goal2 = (CompoundTerm) res.arg(2);
                LinkedList<?> goal1_arg2 = (LinkedList<?>) goal1.arg(2); //Second argument of poss (Z_IN)
                LinkedList<?> goal2_arg3 = (LinkedList<?>) goal2.arg(3); //Third argument of state_update (Z_IN_PR)
                String result = "Input state: ";
                for(int i=0;i<goal1_arg2.size();i++){
                    CompoundTerm fluent = (CompoundTerm) goal1_arg2.get(i);
                    if(i>0){
                        result += ", ";
                    }
                    result += fluent.functor() + "(...)"; 
                }
                result += "\nOutput state: ";
                for(int i=0;i<goal2_arg3.size();i++){
                    CompoundTerm fluent = (CompoundTerm) goal2_arg3.get(i);
                    if(i>0){
                        result += ", ";
                    }
                    result += fluent.functor() + "(...)";   
                }
                JOptionPane.showMessageDialog(this, result, "ECLiPSe Result", JOptionPane.INFORMATION_MESSAGE);
                    // Destroy the Eclipse process
                ((EmbeddedEclipse) engine).destroy();
                System.exit(1);
            }
        } catch (InvalidFormatException | IOException | EclipseException | URISyntaxException  | ParserConfigurationException | SAXException ex) {
            String reason = ex.toString();
            if(reason.startsWith("com.parc")){
                reason = "ECLiPSe syntax error";
            }
            JOptionPane.showMessageDialog(this, reason, "Error", 
                                            JOptionPane.ERROR_MESSAGE);
            //Logger.getLogger(ValidatorGUI.class.getName()).log(Level.SEVERE, null, ex);
            
        }
        
    }//GEN-LAST:event_RunActionPerformed

    private void schemaPathButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_schemaPathButtonActionPerformed
        int returnVal = fc.showOpenDialog(ValidatorGUI.this);
        if (returnVal == JFileChooser.APPROVE_OPTION){
            File schema = fc.getSelectedFile();
            schemaPath.setText(schema.getAbsolutePath());
        }
    }//GEN-LAST:event_schemaPathButtonActionPerformed

    private void filePathButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_filePathButtonActionPerformed
        int returnVal = fc.showOpenDialog(ValidatorGUI.this);
        if (returnVal == JFileChooser.APPROVE_OPTION){
            File file = fc.getSelectedFile();
            filePath.setText(file.getAbsolutePath());
        }
    }//GEN-LAST:event_filePathButtonActionPerformed

    private void EclipseToggleActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_EclipseToggleActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_EclipseToggleActionPerformed

    private void EclipseToggleItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_EclipseToggleItemStateChanged
        if (evt.getStateChange() == ItemEvent.SELECTED){
            //eclipseDirLabel.setEnabled(false);
            //eclipseDir.setEnabled(false);
            //eclipseDirButton.setEnabled(false);
            fluxPath.setEnabled(false);
            fluxPathLabel.setEnabled(false);
            fluxPathButton.setEnabled(false);
            goalPathLabel.setEnabled(false);
            goalPath.setEnabled(false);
            goalPathButton.setEnabled(false);
            goalFormatFLUX.setEnabled(false);
            goalFormatWSSL.setEnabled(false);
        }
        else if(evt.getStateChange() == ItemEvent.DESELECTED){
            //eclipseDirLabel.setEnabled(true);
            //eclipseDir.setEnabled(true);
            //eclipseDirButton.setEnabled(true);
            fluxPath.setEnabled(true);
            fluxPathLabel.setEnabled(true);
            fluxPathButton.setEnabled(true);
            goalPathLabel.setEnabled(true);
            goalPath.setEnabled(true);
            goalPathButton.setEnabled(true);
            goalFormatFLUX.setEnabled(true);
            goalFormatWSSL.setEnabled(true);
        }
    }//GEN-LAST:event_EclipseToggleItemStateChanged

    private void goalPathActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_goalPathActionPerformed
        int returnVal = fc.showOpenDialog(ValidatorGUI.this);
        if (returnVal == JFileChooser.APPROVE_OPTION){
            File file = fc.getSelectedFile();
            goalPath.setText(file.getAbsolutePath());
        }
    }//GEN-LAST:event_goalPathActionPerformed

    private void outputPathActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_outputPathActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_outputPathActionPerformed

    private void goalPathButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_goalPathButtonActionPerformed
        int returnVal = fc.showOpenDialog(ValidatorGUI.this);
        if (returnVal == JFileChooser.APPROVE_OPTION){
            File file = fc.getSelectedFile();
            goalPath.setText(file.getAbsolutePath());
        }
    }//GEN-LAST:event_goalPathButtonActionPerformed

    private void goalFormatFLUXActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_goalFormatFLUXActionPerformed
        //goalPath.setText("wssl/validator/goal.pl");
    }//GEN-LAST:event_goalFormatFLUXActionPerformed

    private void goalFormatWSSLActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_goalFormatWSSLActionPerformed
        //goalPath.setText("wssl/validator/goal.wssl");
    }//GEN-LAST:event_goalFormatWSSLActionPerformed

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        /* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
         * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
         */
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if ("Nimbus".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException | InstantiationException | IllegalAccessException | javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(ValidatorGUI.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        //</editor-fold>

        /* Create and display the form */
        java.awt.EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                new ValidatorGUI().setVisible(true);
            }
        });
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JCheckBox EclipseToggle;
    private javax.swing.JButton Run;
    private javax.swing.JTextField filePath;
    private javax.swing.JButton filePathButton;
    private javax.swing.JLabel filePathLabel;
    private javax.swing.JTextField fluxPath;
    private javax.swing.JButton fluxPathButton;
    private javax.swing.JLabel fluxPathLabel;
    private javax.swing.ButtonGroup goalFormat;
    private javax.swing.JRadioButton goalFormatFLUX;
    private javax.swing.JRadioButton goalFormatWSSL;
    private javax.swing.JTextField goalPath;
    private javax.swing.JButton goalPathButton;
    private javax.swing.JLabel goalPathLabel;
    private javax.swing.JTextField outputPath;
    private javax.swing.JButton outputPathButton;
    private javax.swing.JLabel outputPathLabel;
    private javax.swing.JTextField schemaPath;
    private javax.swing.JButton schemaPathButton;
    private javax.swing.JLabel schemaPathLabel;
    private javax.swing.JSeparator separator;
    // End of variables declaration//GEN-END:variables
    private final javax.swing.JFileChooser fc = new JFileChooser();
    //private File schema = null;
    //private File file = null;
    private String schemaStr = null;
    private String fileStr = null; 
    private String outputStr = null;
    private String goalStr = null;
    private File output = null;
    //private File eclipse = null;
    private File flux = null;
    private static final int INPUT_SIZE = 1000;
}

